// Copyright (c) 2004-2015  Pavel Rychly, Milos Jakubicek

#include <finlib/fromtof.hh>
#include "subcorp.hh"
#include <iostream>
#include <cstdlib>

using namespace std;

// based on RQinNode
class SubcNums: public FastStream {
protected:
    FastStream *inside;
    RangeStream *outside;
    Position infin, outfin;
    bool finished;
    ranges *rng;
    virtual Position locate ()
    {
        if (finished)
            return infin;
        while (true) {
            if (inside->peek() >= infin || outside->peek_beg() >= outfin) {
                finished = true;
                return infin;
            }
            if (rng->beg_at (inside->peek()) < outside->peek_beg()) {
                inside->find (outside->peek_beg());
                continue;
            }
            if (rng->end_at (inside->peek()) > outside->peek_end()) {
                //outside->find_end (inside->peek_end())
                outside->next();
                continue;
            } else
                return inside->peek();
        }
    }
public:
    SubcNums (FastStream *in, ranges *struc, SubCorpus *subc)
    : inside (in), outside (subc->subcorp->whole()), infin (in->final()),
      outfin (outside->final()), finished (false), rng (struc) {locate();}
    virtual ~SubcNums () {delete inside; delete outside;}
    virtual Position next () {inside->next(); return locate();}
    virtual Position peek () {return finished ? infin : inside->peek();}
    virtual void add_labels (Labels &lab) const {NOTIMPLEMENTED}
    virtual Position find (Position pos) {NOTIMPLEMENTED}
    virtual NumOfPos rest_min () {return 0;}
    virtual NumOfPos rest_max ()  {return inside->rest_max();}
    virtual Position final () {return infin;}
};


int main (int argc, char **argv) 
{
    if (argc < 3) {
        cerr << "usage: mknorms CORPUS STRUCT [NORM_STRUCTATTR SUBCORPFILE]\n"
             << "(use - as NORM_STRUCTATTR to use structure size)\n"
             << "example: mknorms mycorp doc wordcount\n";
        return 2;
    }
    try {
        Corpus corp (argv[1]);
        Structure *st = corp.get_struct (argv[2]);
        NumOfPos *norms = new NumOfPos[st->size()];
        string normattrname;
        
        // init norms array
        cerr << "preparing norm counts ..." << endl;
        if (argc >= 4 && argv[3][0] != '-') {
            normattrname = argv[3];
            PosAttr *na = st->get_attr (normattrname);
            TextIterator *it = na->textat (0);
            for (NumOfPos i = 0; i < st->size(); i++)
                norms[i] = atoi (it->next());
            delete it;
        } else {
            for (NumOfPos i = 0; i < st->size(); i++)
                norms[i] = st->rng->end_at(i) - st->rng->beg_at(i);
        }
        SubCorpus *subc = NULL;
        string path = st->conf->opts["PATH"];
        if (argc == 5) {
            subc = new SubCorpus (&corp, argv[4]);
            path = string (argv[4], strlen (argv[4]) - 4) + st->name + '.';
        }

        // save norms
        CorpInfo::VSC::const_iterator sattr = st->conf->attrs.begin();
        for (; sattr != st->conf->attrs.end(); ++sattr) {
            const string &aname = sattr->first;
            if (aname == normattrname)
                continue;
            cerr << "saving norms for " << aname << " ..." << endl;
            ToFile<int64_t> normf (path + aname + ".norm");
            PosAttr *at = st->get_attr (aname);
            int id, id_range = at->id_range();
            for (id = 0; id < id_range; id++) {
                NumOfPos ntotal = 0;
                FastStream *pos = at->id2poss (id);
                if (subc)
                    pos = new SubcNums (pos, st->rng, subc);
                Position p, finval = pos->final();
                while ((p=pos->next()) < finval)
                    ntotal += norms[p];
                delete pos;
                normf.put (ntotal);
            }
        }
    } catch (exception &e) {
        cerr << "mknorms error: " << e.what() << '\n';
        return 1;
    }
    return 0;
}

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
